#!/usr/bin/env python
"""
Function for deterministically generating a single hash for a directory of files,
taking into account only file contents and not filenames.

Usage:

$ dirhash('/path/to/directory', 'md5')

"""


import argparse

import checksumdir

VERSION = checksumdir.__version__


def main():
    parser = argparse.ArgumentParser(description="Determine the hash for directory.")
    parser.add_argument(
        "-v", "--version", action="version", version="checksumdir %s" % VERSION
    )
    parser.add_argument("directory", help="Directory for which to generate hash.")
    parser.add_argument(
        "-a", "--algorithm", choices=("md5", "sha1", "sha256", "sha512"), default="md5"
    )
    parser.add_argument(
        "-e", "--excluded-files", nargs="+", help="List of excluded files."
    )
    parser.add_argument(
        "-i",
        "--ignore-hidden",
        action="store_true",
        default=False,
        help="Ignore hidden files",
    )
    parser.add_argument(
        "-f",
        "--follow-links",
        action="store_true",
        default=False,
        help="Follow soft links",
    )
    parser.add_argument(
        "-x",
        "--excluded-extensions",
        nargs="+",
        help="List of excluded file extensions.",
    )

    args = parser.parse_args()
    print(
        checksumdir.dirhash(
            dirname=args.directory,
            hashfunc=args.algorithm,
            excluded_files=args.excluded_files,
            ignore_hidden=args.ignore_hidden,
            followlinks=args.follow_links,
            excluded_extensions=args.excluded_extensions,
        )
    )


if __name__ == "__main__":
    main()



"""
Function for deterministically creating a single hash for a directory of files,
taking into account only file contents and not filenames.

Usage:

from checksumdir import dirhash

dirhash('/path/to/directory', 'md5')

"""

import hashlib
import os
import re

import pkg_resources

__version__ = pkg_resources.require("checksumdir")[0].version

HASH_FUNCS = {
    "md5": hashlib.md5,
    "sha1": hashlib.sha1,
    "sha256": hashlib.sha256,
    "sha512": hashlib.sha512,
}


def dirhash(
    dirname,
    hashfunc="md5",
    excluded_files=None,
    ignore_hidden=False,
    followlinks=False,
    excluded_extensions=None,
):
    hash_func = HASH_FUNCS.get(hashfunc)
    if not hash_func:
        raise NotImplementedError("{} not implemented.".format(hashfunc))

    if not excluded_files:
        excluded_files = []

    if not excluded_extensions:
        excluded_extensions = []

    if not os.path.isdir(dirname):
        raise TypeError("{} is not a directory.".format(dirname))

    hashvalues = []
    for root, dirs, files in os.walk(dirname, topdown=True, followlinks=followlinks):
        if ignore_hidden and re.search(r"/\.", root):
            continue

        dirs.sort()
        files.sort()

        for fname in files:
            if ignore_hidden and fname.startswith("."):
                continue

            if fname.split(".")[-1:][0] in excluded_extensions:
                continue

            if fname in excluded_files:
                continue

            hashvalues.append(_filehash(os.path.join(root, fname), hash_func))

    return _reduce_hash(hashvalues, hash_func)


def _filehash(filepath, hashfunc):
    hasher = hashfunc()
    blocksize = 64 * 1024

    if not os.path.exists(filepath):
        return hasher.hexdigest()

    with open(filepath, "rb") as fp:
        while True:
            data = fp.read(blocksize)
            if not data:
                break
            hasher.update(data)
    return hasher.hexdigest()


def _reduce_hash(hashlist, hashfunc):
    hasher = hashfunc()
    for hashvalue in sorted(hashlist):
        hasher.update(hashvalue.encode("utf-8"))
    return hasher.hexdigest()



from pathlib import Path
from typing import Callable, List, Optional, Union

def dirhash(
    dirname: Union[str, Path],
    hashfunc: str = "md5",
    excluded_files: Optional[List[str]] = None,
    ignore_hidden: bool = False,
    followlinks: bool = False,
    excluded_extensions: Optional[List[str]] = None,
) -> str: ...
def _filehash(filepath: str, hashfunc: Callable) -> str: ...
def _reduce_hash(hashlist: List[str], hashfunc: Callable) -> str: ...

__version__: str



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['checksumdir']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['checksumdir = checksumdir.cli:main']}

setup_kwargs = {
    'name': 'checksumdir',
    'version': '1.1.9',
    'description': 'Compute a single hash of the file contents of a directory.',
    'long_description': "***********\nChecksumdir\n***********\n\n|badge1| |badge2|\n\n\n.. |badge1| image:: https://img.shields.io/pypi/dm/checksumdir   \n    :alt: PyPI - Downloads\n    :target: https://pypi.org/project/checksumdir/\n\n.. |badge2| image:: https://badge.fury.io/py/checksumdir.svg\n    :target: https://pypi.org/project/checksumdir/\n\nA simple module for creating a single hash for a directory of files, with file contents;\nignoring any metadata such as file name.  Options exist to also exclude specific files\nor files with specific extensions.\n\n=====\nUsage\n=====\n\n.. code-block:: python\n\n    from checksumdir import dirhash\n\n    directory  = '/path/to/directory/'\n    md5hash    = dirhash(directory, 'md5')\n    sha1hash   = dirhash(directory, 'sha1', excluded_files=['package.json'])\n    sha256hash = dirhash(directory, 'sha256', excluded_extensions=['pyc'])\n\n\nOr to use the CLI:\n\n.. code-block:: bash\n\n    # Defaults to md5.\n    $ checksumdir /path/to/directory\n\n    # Create sha1 hash:\n    $ checksumdir -a sha1 /path/to/directory\n\n    # Exclude files:\n    $ checksumdir -e <files> /path/to/directory\n\n    # Exclude files with specific extensions:\n    $ checksumdir -x <extensions> /path/to/directory\n\n    # Follow soft links:\n    $ checksumdir --follow-links /path/to/directory\n",
    'author': 'Tom McCarthy',
    'author_email': 'tmac.se@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'http://github.com/cakepietoast/checksumdir',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
